"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MarkdownTheme = void 0;
const typedoc_1 = require("typedoc");
const custom_maps_1 = require("../plugin/options/custom-maps");
const utils_1 = require("../support/utils");
const navigation_context_1 = require("./navigation-context");
const render_context_1 = require("./render-context");
const urls_context_1 = require("./urls-context");
/**
 * This is in-built MarkdownTheme which extends TypeDocs Theme class.
 * This follows the implementation of TypeDoc's [DefaultTheme](https://typedoc.org/api/classes/DefaultThemeRender.html).
 *
 * The {@link render } and {@link getUrls} methods is where the work happens.
 */
class MarkdownTheme extends typedoc_1.Theme {
    /**
     *
     * @param renderer The TypeDoc renderer instance the theme is attached to.
     */
    constructor(renderer) {
        super(renderer);
        this.readmeTemplate = (pageEvent) => {
            return this.getRenderContext(pageEvent).readmeTemplate(pageEvent);
        };
        this.projectTemplate = (pageEvent) => {
            return this.getRenderContext(pageEvent).projectTemplate(pageEvent);
        };
        this.reflectionTemplate = (pageEvent) => {
            return this.getRenderContext(pageEvent).reflectionTemplate(pageEvent);
        };
        this.memberTemplate = (pageEvent) => {
            return this.getRenderContext(pageEvent).memberTemplate(pageEvent);
        };
    }
    getRenderContext(pageEvent) {
        return new render_context_1.MarkdownThemeRenderContext(pageEvent, this.application.options);
    }
    getNavigationContext() {
        return new navigation_context_1.NavigationContext(this, this.application.options.getRawValues());
    }
    getUrlsContext(project) {
        return new urls_context_1.UrlsContext(this, project, this.application.renderer, this.application.options);
    }
    /**
     * Renders a template and page model to a string.
     */
    render(page, template) {
        const contents = template(page);
        return this.hasToc(page) ? this.insertToc(page, contents) : contents;
    }
    getUrls(project) {
        return this.getUrlsContext(project).getUrls();
    }
    getNavigation(project) {
        return this.getNavigationContext().getNavigation(project);
    }
    hasToc(page) {
        var _a;
        if (this.application.options.getValue('hideInPageTOC')) {
            return false;
        }
        const reflection = page.model;
        return (reflection.kindOf([
            typedoc_1.ReflectionKind.Project,
            typedoc_1.ReflectionKind.Module,
            typedoc_1.ReflectionKind.Namespace,
            typedoc_1.ReflectionKind.Enum,
            typedoc_1.ReflectionKind.Class,
            typedoc_1.ReflectionKind.Interface,
        ]) &&
            ((_a = reflection.groups) === null || _a === void 0 ? void 0 : _a.some((group) => !group.allChildrenHaveOwnDocument())));
    }
    insertToc(page, contents) {
        var _a;
        const regXHeader = /(?<flag>#{2,6})\s+(?<content>.+)/g;
        const anchors = [];
        const toc = (_a = Array.from(contents === null || contents === void 0 ? void 0 : contents.matchAll(regXHeader))) === null || _a === void 0 ? void 0 : _a.map(({ groups: { flag, content } }) => {
            return {
                heading: flag.length,
                content,
            };
        }).map((item) => {
            var _a;
            anchors.push(item.content);
            const count = (_a = anchors === null || anchors === void 0 ? void 0 : anchors.filter((id) => id === item.content)) === null || _a === void 0 ? void 0 : _a.length;
            return { ...item, count };
        }).filter((item) => item.heading < 4).map((item) => {
            const spaces = Array.from({ length: item.heading })
                .map((_, i) => (i > 1 ? '  ' : ''))
                .join('');
            const urlParts = page === null || page === void 0 ? void 0 : page.url.split('/');
            const urlBase = urlParts[urlParts.length - 1];
            return `${spaces}- [${item.content}](${encodeURI(urlBase)}#${(0, utils_1.slugify)(item.content).toLowerCase()}${item.count > 1 ? `-${item.count - 1}` : ''})`;
        }).join('\n');
        const contentToLines = contents === null || contents === void 0 ? void 0 : contents.split('\n');
        const firstHeadingIndex = contentToLines === null || contentToLines === void 0 ? void 0 : contentToLines.findIndex((line) => line.startsWith('##'));
        if (firstHeadingIndex && firstHeadingIndex > 0) {
            contentToLines === null || contentToLines === void 0 ? void 0 : contentToLines.splice(firstHeadingIndex, 0, `\n\n## Contents\n\n${toc}\n\n`);
        }
        return contentToLines === null || contentToLines === void 0 ? void 0 : contentToLines.join('\n');
    }
    /**
     * Returns the template mapping for a given reflection kind
     * @param kind
     */
    getTemplateMapping(kind, outputFileStrategyOverride) {
        const outputFileStrategy = outputFileStrategyOverride ||
            this.application.options.getValue('outputFileStrategy');
        const getDirectoryName = (reflectionKind) => {
            const pluralString = typedoc_1.ReflectionKind.pluralString(reflectionKind);
            return (0, utils_1.slugify)(pluralString).toLowerCase();
        };
        const membersWithOwnFile = this.application.options.getValue('membersWithOwnFile');
        const mappings = {
            [typedoc_1.ReflectionKind.Module]: {
                template: this.reflectionTemplate,
                directory: null,
                kind: typedoc_1.ReflectionKind.Module,
            },
            [typedoc_1.ReflectionKind.Namespace]: {
                template: this.reflectionTemplate,
                directory: getDirectoryName(typedoc_1.ReflectionKind.Namespace),
                kind: typedoc_1.ReflectionKind.Namespace,
            },
        };
        if (outputFileStrategy === custom_maps_1.OutputFileStrategy.Members &&
            (membersWithOwnFile === null || membersWithOwnFile === void 0 ? void 0 : membersWithOwnFile.includes(typedoc_1.ReflectionKind[typedoc_1.ReflectionKind.Class]))) {
            mappings[typedoc_1.ReflectionKind.Class] = {
                template: this.reflectionTemplate,
                directory: getDirectoryName(typedoc_1.ReflectionKind.Class),
                kind: typedoc_1.ReflectionKind.Class,
            };
        }
        if (outputFileStrategy === custom_maps_1.OutputFileStrategy.Members &&
            (membersWithOwnFile === null || membersWithOwnFile === void 0 ? void 0 : membersWithOwnFile.includes(typedoc_1.ReflectionKind[typedoc_1.ReflectionKind.Interface]))) {
            mappings[typedoc_1.ReflectionKind.Interface] = {
                isLeaf: false,
                template: this.reflectionTemplate,
                directory: getDirectoryName(typedoc_1.ReflectionKind.Interface),
                kind: typedoc_1.ReflectionKind.Interface,
            };
        }
        if (outputFileStrategy === custom_maps_1.OutputFileStrategy.Members &&
            (membersWithOwnFile === null || membersWithOwnFile === void 0 ? void 0 : membersWithOwnFile.includes(typedoc_1.ReflectionKind[typedoc_1.ReflectionKind.Enum]))) {
            mappings[typedoc_1.ReflectionKind.Enum] = {
                template: this.reflectionTemplate,
                directory: getDirectoryName(typedoc_1.ReflectionKind.Enum),
                kind: typedoc_1.ReflectionKind.Enum,
            };
        }
        if (outputFileStrategy === custom_maps_1.OutputFileStrategy.Members &&
            (membersWithOwnFile === null || membersWithOwnFile === void 0 ? void 0 : membersWithOwnFile.includes(typedoc_1.ReflectionKind[typedoc_1.ReflectionKind.Function]))) {
            mappings[typedoc_1.ReflectionKind.Function] = {
                template: this.memberTemplate,
                directory: getDirectoryName(typedoc_1.ReflectionKind.Function),
                kind: typedoc_1.ReflectionKind.Function,
            };
        }
        if (outputFileStrategy === custom_maps_1.OutputFileStrategy.Members &&
            (membersWithOwnFile === null || membersWithOwnFile === void 0 ? void 0 : membersWithOwnFile.includes(typedoc_1.ReflectionKind[typedoc_1.ReflectionKind.TypeAlias]))) {
            mappings[typedoc_1.ReflectionKind.TypeAlias] = {
                template: this.memberTemplate,
                directory: getDirectoryName(typedoc_1.ReflectionKind.TypeAlias),
                kind: typedoc_1.ReflectionKind.TypeAlias,
            };
        }
        if (outputFileStrategy === custom_maps_1.OutputFileStrategy.Members &&
            (membersWithOwnFile === null || membersWithOwnFile === void 0 ? void 0 : membersWithOwnFile.includes(typedoc_1.ReflectionKind[typedoc_1.ReflectionKind.Variable]))) {
            mappings[typedoc_1.ReflectionKind.Variable] = {
                template: this.memberTemplate,
                directory: getDirectoryName(typedoc_1.ReflectionKind.Variable),
                kind: typedoc_1.ReflectionKind.Variable,
            };
        }
        return mappings[kind];
    }
}
exports.MarkdownTheme = MarkdownTheme;
